@@ -25,10 +25,11 @@ pub use super::ffi::cl_dx9_media_sharing::{
 pub use super::ffi::cl_egl::{CL_EGL_RESOURCE_NOT_ACQUIRED_KHR, CL_INVALID_EGL_OBJECT_KHR};
 pub use super::ffi::cl_ext::{
     CL_ACCELERATOR_TYPE_NOT_SUPPORTED_INTEL, CL_COMMAND_TERMINATED_ITSELF_WITH_FAILURE_ARM,
-    CL_CONTEXT_TERMINATED_KHR, CL_DEVICE_PARTITION_FAILED_EXT,
+    CL_CONTEXT_TERMINATED_KHR, CL_DEVICE_PARTITION_FAILED_EXT, CL_INCOMPATIBLE_COMMAND_QUEUE_KHR,
     CL_INVALID_ACCELERATOR_DESCRIPTOR_INTEL, CL_INVALID_ACCELERATOR_INTEL,
-    CL_INVALID_ACCELERATOR_TYPE_INTEL, CL_INVALID_PARTITION_COUNT_EXT,
-    CL_INVALID_PARTITION_NAME_EXT,
+    CL_INVALID_ACCELERATOR_TYPE_INTEL, CL_INVALID_COMMAND_BUFFER_KHR,
+    CL_INVALID_PARTITION_COUNT_EXT, CL_INVALID_PARTITION_NAME_EXT, CL_INVALID_SEMAPHORE_KHR,
+    CL_INVALID_SYNC_POINT_WAIT_LIST_KHR,
 };
 pub use cl_sys::{
     cl_int, CL_BUILD_PROGRAM_FAILURE, CL_COMPILER_NOT_AVAILABLE, CL_COMPILE_PROGRAM_FAILURE,
@@ -156,6 +157,11 @@ pub fn error_text(error_code: cl_int) -> &'static str {
         }
 
         CL_CONTEXT_TERMINATED_KHR => "CL_CONTEXT_TERMINATED_KHR",
+        CL_INVALID_SEMAPHORE_KHR => "CL_INVALID_SEMAPHORE_KHR",
+
+        CL_INVALID_COMMAND_BUFFER_KHR => "CL_INVALID_COMMAND_BUFFER_KHR",
+        CL_INVALID_SYNC_POINT_WAIT_LIST_KHR => "CL_INVALID_SYNC_POINT_WAIT_LIST_KHR",
+        CL_INCOMPATIBLE_COMMAND_QUEUE_KHR => "CL_INCOMPATIBLE_COMMAND_QUEUE_KHR",
 
         _ => "UNKNOWN_ERROR",
     }
