@@ -13,6 +13,7 @@
 // limitations under the License.
 
 //! OpenCL extensions that don't have external (OpenGL, D3D) dependencies.
+//! See: [OpenCL Extension Specification](https://www.khronos.org/registry/OpenCL/specs/3.0-unified/html/OpenCL_Ext.html)
 
 #![allow(non_camel_case_types)]
 #![allow(clippy::not_unsafe_ptr_arg_deref)]
@@ -33,6 +34,453 @@ use std::mem;
 #[allow(unused_imports)]
 use std::ptr;
 
+#[cfg(feature = "cl_khr_command_buffer")]
+pub fn create_command_buffer_khr(
+    queues: &[cl_command_queue],
+    properties: *const cl_command_buffer_properties_khr,
+) -> Result<cl_command_buffer_khr, cl_int> {
+    let mut status: cl_int = CL_INVALID_VALUE;
+    let buffer = unsafe {
+        clCreateCommandBufferKHR(
+            queues.len() as cl_uint,
+            queues.as_ptr(),
+            properties,
+            &mut status,
+        )
+    };
+    if CL_SUCCESS != status {
+        Err(status)
+    } else {
+        Ok(buffer)
+    }
+}
+
+#[cfg(feature = "cl_khr_command_buffer")]
+pub fn finalize_command_buffer_khr(command_buffer: cl_command_buffer_khr) -> Result<(), cl_int> {
+    let status: cl_int = unsafe { clFinalizeCommandBufferKHR(command_buffer) };
+    if CL_SUCCESS != status {
+        Err(status)
+    } else {
+        Ok(())
+    }
+}
+
+#[cfg(feature = "cl_khr_command_buffer")]
+pub fn retain_command_buffer_khr(command_buffer: cl_command_buffer_khr) -> Result<(), cl_int> {
+    let status: cl_int = unsafe { clRetainCommandBufferKHR(command_buffer) };
+    if CL_SUCCESS != status {
+        Err(status)
+    } else {
+        Ok(())
+    }
+}
+
+#[cfg(feature = "cl_khr_command_buffer")]
+pub fn release_command_buffer_khr(command_buffer: cl_command_buffer_khr) -> Result<(), cl_int> {
+    let status: cl_int = unsafe { clReleaseCommandBufferKHR(command_buffer) };
+    if CL_SUCCESS != status {
+        Err(status)
+    } else {
+        Ok(())
+    }
+}
+
+#[cfg(feature = "cl_khr_command_buffer")]
+pub fn enqueue_command_buffer_khr(
+    num_queues: cl_uint,
+    queues: *mut cl_command_queue,
+    command_buffer: cl_command_buffer_khr,
+    num_events_in_wait_list: cl_uint,
+    event_wait_list: *const cl_event,
+) -> Result<cl_event, cl_int> {
+    let mut event: cl_event = ptr::null_mut();
+    let status: cl_int = unsafe {
+        clEnqueueCommandBufferKHR(
+            num_queues,
+            queues,
+            command_buffer,
+            num_events_in_wait_list,
+            event_wait_list,
+            &mut event,
+        )
+    };
+    if CL_SUCCESS != status {
+        Err(status)
+    } else {
+        Ok(event)
+    }
+}
+
+#[cfg(feature = "cl_khr_command_buffer")]
+pub fn command_barrier_with_wait_list_khr(
+    command_buffer: cl_command_buffer_khr,
+    command_queue: cl_command_queue,
+    sync_point_wait_list: &[cl_sync_point_khr],
+    sync_point: *mut cl_sync_point_khr,
+    mutable_handle: *mut cl_mutable_command_khr,
+) -> Result<(), cl_int> {
+    let status: cl_int = unsafe {
+        clCommandBarrierWithWaitListKHR(
+            command_buffer,
+            command_queue,
+            sync_point_wait_list.len() as cl_uint,
+            sync_point_wait_list.as_ptr(),
+            sync_point,
+            mutable_handle,
+        )
+    };
+    if CL_SUCCESS != status {
+        Err(status)
+    } else {
+        Ok(())
+    }
+}
+
+#[cfg(feature = "cl_khr_command_buffer")]
+pub fn command_copy_buffer_khr(
+    command_buffer: cl_command_buffer_khr,
+    command_queue: cl_command_queue,
+    src_buffer: cl_mem,
+    dst_buffer: cl_mem,
+    src_offset: size_t,
+    dst_offset: size_t,
+    size: size_t,
+    sync_point_wait_list: &[cl_sync_point_khr],
+    sync_point: *mut cl_sync_point_khr,
+    mutable_handle: *mut cl_mutable_command_khr,
+) -> Result<(), cl_int> {
+    let status: cl_int = unsafe {
+        clCommandCopyBufferKHR(
+            command_buffer,
+            command_queue,
+            src_buffer,
+            dst_buffer,
+            src_offset,
+            dst_offset,
+            size,
+            sync_point_wait_list.len() as cl_uint,
+            sync_point_wait_list.as_ptr(),
+            sync_point,
+            mutable_handle,
+        )
+    };
+    if CL_SUCCESS != status {
+        Err(status)
+    } else {
+        Ok(())
+    }
+}
+
+#[cfg(feature = "cl_khr_command_buffer")]
+pub fn command_copy_buffer_rect_khr(
+    command_buffer: cl_command_buffer_khr,
+    command_queue: cl_command_queue,
+    src_buffer: cl_mem,
+    dst_buffer: cl_mem,
+    src_origin: *const size_t,
+    dst_origin: *const size_t,
+    region: *const size_t,
+    src_row_pitch: size_t,
+    src_slice_pitch: size_t,
+    dst_row_pitch: size_t,
+    dst_slice_pitch: size_t,
+    sync_point_wait_list: &[cl_sync_point_khr],
+    sync_point: *mut cl_sync_point_khr,
+    mutable_handle: *mut cl_mutable_command_khr,
+) -> Result<(), cl_int> {
+    let status: cl_int = unsafe {
+        clCommandCopyBufferRectKHR(
+            command_buffer,
+            command_queue,
+            src_buffer,
+            dst_buffer,
+            src_origin,
+            dst_origin,
+            region,
+            src_row_pitch,
+            src_slice_pitch,
+            dst_row_pitch,
+            dst_slice_pitch,
+            sync_point_wait_list.len() as cl_uint,
+            sync_point_wait_list.as_ptr(),
+            sync_point,
+            mutable_handle,
+        )
+    };
+    if CL_SUCCESS != status {
+        Err(status)
+    } else {
+        Ok(())
+    }
+}
+
+#[cfg(feature = "cl_khr_command_buffer")]
+pub fn command_copy_buffer_to_image_khr(
+    command_buffer: cl_command_buffer_khr,
+    command_queue: cl_command_queue,
+    src_buffer: cl_mem,
+    dst_image: cl_mem,
+    src_offset: size_t,
+    dst_origin: *const size_t,
+    region: *const size_t,
+    sync_point_wait_list: &[cl_sync_point_khr],
+    sync_point: *mut cl_sync_point_khr,
+    mutable_handle: *mut cl_mutable_command_khr,
+) -> Result<(), cl_int> {
+    let status: cl_int = unsafe {
+        clCommandCopyBufferToImageKHR(
+            command_buffer,
+            command_queue,
+            src_buffer,
+            dst_image,
+            src_offset,
+            dst_origin,
+            region,
+            sync_point_wait_list.len() as cl_uint,
+            sync_point_wait_list.as_ptr(),
+            sync_point,
+            mutable_handle,
+        )
+    };
+    if CL_SUCCESS != status {
+        Err(status)
+    } else {
+        Ok(())
+    }
+}
+
+#[cfg(feature = "cl_khr_command_buffer")]
+pub fn command_copy_image_khr(
+    command_buffer: cl_command_buffer_khr,
+    command_queue: cl_command_queue,
+    src_image: cl_mem,
+    dst_image: cl_mem,
+    src_origin: *const size_t,
+    dst_origin: *const size_t,
+    region: *const size_t,
+    sync_point_wait_list: &[cl_sync_point_khr],
+    sync_point: *mut cl_sync_point_khr,
+    mutable_handle: *mut cl_mutable_command_khr,
+) -> Result<(), cl_int> {
+    let status: cl_int = unsafe {
+        clCommandCopyImageKHR(
+            command_buffer,
+            command_queue,
+            src_image,
+            dst_image,
+            src_origin,
+            dst_origin,
+            region,
+            sync_point_wait_list.len() as cl_uint,
+            sync_point_wait_list.as_ptr(),
+            sync_point,
+            mutable_handle,
+        )
+    };
+    if CL_SUCCESS != status {
+        Err(status)
+    } else {
+        Ok(())
+    }
+}
+
+#[cfg(feature = "cl_khr_command_buffer")]
+pub fn command_copy_image_to_buffer_khr(
+    command_buffer: cl_command_buffer_khr,
+    command_queue: cl_command_queue,
+    src_image: cl_mem,
+    dst_buffer: cl_mem,
+    src_origin: *const size_t,
+    region: *const size_t,
+    dst_offset: size_t,
+    sync_point_wait_list: &[cl_sync_point_khr],
+    sync_point: *mut cl_sync_point_khr,
+    mutable_handle: *mut cl_mutable_command_khr,
+) -> Result<(), cl_int> {
+    let status: cl_int = unsafe {
+        clCommandCopyImageToBufferKHR(
+            command_buffer,
+            command_queue,
+            src_image,
+            dst_buffer,
+            src_origin,
+            region,
+            dst_offset,
+            sync_point_wait_list.len() as cl_uint,
+            sync_point_wait_list.as_ptr(),
+            sync_point,
+            mutable_handle,
+        )
+    };
+    if CL_SUCCESS != status {
+        Err(status)
+    } else {
+        Ok(())
+    }
+}
+
+#[cfg(feature = "cl_khr_command_buffer")]
+pub fn command_fill_buffer_khr(
+    command_buffer: cl_command_buffer_khr,
+    command_queue: cl_command_queue,
+    buffer: cl_mem,
+    pattern: *const c_void,
+    pattern_size: size_t,
+    offset: size_t,
+    size: size_t,
+    sync_point_wait_list: &[cl_sync_point_khr],
+    sync_point: *mut cl_sync_point_khr,
+    mutable_handle: *mut cl_mutable_command_khr,
+) -> Result<(), cl_int> {
+    let status: cl_int = unsafe {
+        clCommandFillBufferKHR(
+            command_buffer,
+            command_queue,
+            buffer,
+            pattern,
+            pattern_size,
+            offset,
+            size,
+            sync_point_wait_list.len() as cl_uint,
+            sync_point_wait_list.as_ptr(),
+            sync_point,
+            mutable_handle,
+        )
+    };
+    if CL_SUCCESS != status {
+        Err(status)
+    } else {
+        Ok(())
+    }
+}
+
+#[cfg(feature = "cl_khr_command_buffer")]
+pub fn command_fill_image_khr(
+    command_buffer: cl_command_buffer_khr,
+    command_queue: cl_command_queue,
+    image: cl_mem,
+    fill_color: *const c_void,
+    origin: *const size_t,
+    region: *const size_t,
+    sync_point_wait_list: &[cl_sync_point_khr],
+    sync_point: *mut cl_sync_point_khr,
+    mutable_handle: *mut cl_mutable_command_khr,
+) -> Result<(), cl_int> {
+    let status: cl_int = unsafe {
+        clCommandFillImageKHR(
+            command_buffer,
+            command_queue,
+            image,
+            fill_color,
+            origin,
+            region,
+            sync_point_wait_list.len() as cl_uint,
+            sync_point_wait_list.as_ptr(),
+            sync_point,
+            mutable_handle,
+        )
+    };
+    if CL_SUCCESS != status {
+        Err(status)
+    } else {
+        Ok(())
+    }
+}
+
+#[cfg(feature = "cl_khr_command_buffer")]
+pub fn command_nd_range_kernel_khr(
+    command_buffer: cl_command_buffer_khr,
+    command_queue: cl_command_queue,
+    properties: *const cl_ndrange_kernel_command_properties_khr,
+    kernel: cl_kernel,
+    work_dim: cl_uint,
+    global_work_offset: *const size_t,
+    global_work_size: *const size_t,
+    local_work_size: *const size_t,
+    sync_point_wait_list: &[cl_sync_point_khr],
+    sync_point: *mut cl_sync_point_khr,
+    mutable_handle: *mut cl_mutable_command_khr,
+) -> Result<(), cl_int> {
+    let status: cl_int = unsafe {
+        clCommandNDRangeKernelKHR(
+            command_buffer,
+            command_queue,
+            properties,
+            kernel,
+            work_dim,
+            global_work_offset,
+            global_work_size,
+            local_work_size,
+            sync_point_wait_list.len() as cl_uint,
+            sync_point_wait_list.as_ptr(),
+            sync_point,
+            mutable_handle,
+        )
+    };
+    if CL_SUCCESS != status {
+        Err(status)
+    } else {
+        Ok(())
+    }
+}
+
+#[cfg(feature = "cl_khr_command_buffer")]
+pub fn get_command_buffer_data_khr(
+    command_buffer: cl_command_buffer_khr,
+    param_name: cl_command_buffer_info_khr,
+) -> Result<Vec<u8>, cl_int> {
+    api_info_size!(get_size, clGetCommandBufferInfoKHR);
+    let size = get_size(command_buffer, param_name)?;
+    api_info_vector!(get_vector, u8, clGetCommandBufferInfoKHR);
+    get_vector(command_buffer, param_name, size)
+}
+
+#[cfg(feature = "cl_khr_command_buffer")]
+pub fn get_command_buffer_info_khr(
+    command_queue: cl_command_buffer_khr,
+    param_name: cl_command_buffer_info_khr,
+) -> Result<InfoType, cl_int> {
+    match param_name {
+        CL_COMMAND_BUFFER_NUM_QUEUES_KHR
+        | CL_COMMAND_BUFFER_REFERENCE_COUNT_KHR
+        | CL_COMMAND_BUFFER_STATE_KHR => {
+            api_info_value!(get_value, cl_uint, clGetCommandBufferInfoKHR);
+            Ok(InfoType::Uint(get_value(command_queue, param_name)?))
+        }
+
+        CL_COMMAND_BUFFER_PROPERTIES_ARRAY_KHR => {
+            api_info_size!(get_size, clGetCommandBufferInfoKHR);
+            api_info_vector!(
+                get_vec,
+                cl_command_buffer_properties_khr,
+                clGetCommandBufferInfoKHR
+            );
+            let size = get_size(command_queue, param_name)?;
+            Ok(InfoType::VecUlong(get_vec(
+                command_queue,
+                param_name,
+                size,
+            )?))
+        }
+
+        CL_COMMAND_BUFFER_QUEUES_KHR => {
+            api_info_size!(get_size, clGetCommandBufferInfoKHR);
+            api_info_vector!(get_vec, intptr_t, clGetCommandBufferInfoKHR);
+            let size = get_size(command_queue, param_name)?;
+            Ok(InfoType::VecIntPtr(get_vec(
+                command_queue,
+                param_name,
+                size,
+            )?))
+        }
+
+        _ => Ok(InfoType::VecUchar(get_command_buffer_data_khr(
+            command_queue,
+            param_name,
+        )?)),
+    }
+}
+
 #[cfg(feature = "cl_apple_setmemobjectdestructor")]
 pub fn set_mem_object_destructor_apple(
     memobj: cl_mem,
@@ -395,6 +843,201 @@ pub fn get_kernel_suggested_local_work_size_khr(
     }
 }
 
+#[cfg(feature = "cl_khr_external_memory")]
+pub fn enqueue_acquire_external_mem_objects_khr(
+    command_queue: cl_command_queue,
+    num_mem_objects: cl_uint,
+    mem_objects: *const cl_mem,
+    num_events_in_wait_list: cl_uint,
+    event_wait_list: *const cl_event,
+) -> Result<cl_event, cl_int> {
+    let mut event: cl_event = ptr::null_mut();
+    let status: cl_int = unsafe {
+        clEnqueueAcquireExternalMemObjectsKHR(
+            command_queue,
+            num_mem_objects,
+            mem_objects,
+            num_events_in_wait_list,
+            event_wait_list,
+            &mut event,
+        )
+    };
+    if CL_SUCCESS != status {
+        Err(status)
+    } else {
+        Ok(event)
+    }
+}
+
+#[cfg(feature = "cl_khr_external_memory")]
+pub fn enqueue_release_external_mem_objects_khr(
+    command_queue: cl_command_queue,
+    num_mem_objects: cl_uint,
+    mem_objects: *const cl_mem,
+    num_events_in_wait_list: cl_uint,
+    event_wait_list: *const cl_event,
+) -> Result<cl_event, cl_int> {
+    let mut event: cl_event = ptr::null_mut();
+    let status: cl_int = unsafe {
+        clEnqueueReleaseExternalMemObjectsKHR(
+            command_queue,
+            num_mem_objects,
+            mem_objects,
+            num_events_in_wait_list,
+            event_wait_list,
+            &mut event,
+        )
+    };
+    if CL_SUCCESS != status {
+        Err(status)
+    } else {
+        Ok(event)
+    }
+}
+
+#[cfg(feature = "cl_khr_external_semaphore")]
+pub fn get_semaphore_handle_for_type_khr(
+    sema_object: cl_semaphore_khr,
+    device: cl_device_id,
+    handle_type: cl_external_semaphore_handle_type_khr,
+) -> Result<cl_semaphore_khr, cl_int> {
+    // Get the size of the information.
+    let mut size: size_t = 0;
+    let status: cl_int = unsafe {
+        clGetSemaphoreHandleForTypeKHR(
+            sema_object,
+            device,
+            handle_type,
+            0,
+            ptr::null_mut(),
+            &mut size,
+        )
+    };
+    if CL_SUCCESS != status {
+        Err(status)
+    } else {
+        let mut data: cl_semaphore_khr = ptr::null_mut();
+        let data_ptr: *mut cl_semaphore_khr = &mut data;
+        let status: cl_int = unsafe {
+            clGetSemaphoreHandleForTypeKHR(
+                sema_object,
+                device,
+                handle_type,
+                size,
+                data_ptr as *mut c_void,
+                ptr::null_mut(),
+            )
+        };
+        if CL_SUCCESS != status {
+            Err(status)
+        } else {
+            Ok(data)
+        }
+    }
+}
+
+#[cfg(feature = "cl_khr_semaphore")]
+pub fn create_semaphore_with_properties_khr(
+    context: cl_context,
+    sema_props: *const cl_semaphore_properties_khr,
+) -> Result<cl_semaphore_khr, cl_int> {
+    let mut status: cl_int = CL_INVALID_VALUE;
+    let semaphore: cl_semaphore_khr =
+        unsafe { clCreateSemaphoreWithPropertiesKHR(context, sema_props, &mut status) };
+    if CL_SUCCESS != status {
+        Err(status)
+    } else {
+        Ok(semaphore)
+    }
+}
+
+#[cfg(feature = "cl_khr_semaphore")]
+pub fn enqueue_wait_semaphores_khr(
+    command_queue: cl_command_queue,
+    num_sema_objects: cl_uint,
+    sema_objects: *const cl_semaphore_khr,
+    sema_payload_list: *const cl_semaphore_payload_khr,
+    num_events_in_wait_list: cl_uint,
+    event_wait_list: *const cl_event,
+) -> Result<cl_event, cl_int> {
+    let mut event: cl_event = ptr::null_mut();
+    let status: cl_int = unsafe {
+        clEnqueueWaitSemaphoresKHR(
+            command_queue,
+            num_sema_objects,
+            sema_objects,
+            sema_payload_list,
+            num_events_in_wait_list,
+            event_wait_list,
+            &mut event,
+        )
+    };
+    if CL_SUCCESS != status {
+        Err(status)
+    } else {
+        Ok(event)
+    }
+}
+
+#[cfg(feature = "cl_khr_semaphore")]
+pub fn enqueue_signal_semaphores_khr(
+    command_queue: cl_command_queue,
+    num_sema_objects: cl_uint,
+    sema_objects: *const cl_semaphore_khr,
+    sema_payload_list: *const cl_semaphore_payload_khr,
+    num_events_in_wait_list: cl_uint,
+    event_wait_list: *const cl_event,
+) -> Result<cl_event, cl_int> {
+    let mut event: cl_event = ptr::null_mut();
+    let status: cl_int = unsafe {
+        clEnqueueSignalSemaphoresKHR(
+            command_queue,
+            num_sema_objects,
+            sema_objects,
+            sema_payload_list,
+            num_events_in_wait_list,
+            event_wait_list,
+            &mut event,
+        )
+    };
+    if CL_SUCCESS != status {
+        Err(status)
+    } else {
+        Ok(event)
+    }
+}
+
+#[cfg(feature = "cl_khr_semaphore")]
+pub fn get_semaphore_info_khr(
+    sema_object: cl_semaphore_khr,
+    param_name: cl_semaphore_info_khr,
+) -> Result<Vec<u8>, cl_int> {
+    api_info_size!(get_size, clGetSemaphoreInfoKHR);
+    let size = get_size(sema_object, param_name)?;
+    api_info_vector!(get_vector, u8, clGetSemaphoreInfoKHR);
+    get_vector(sema_object, param_name, size)
+}
+
+#[cfg(feature = "cl_khr_semaphore")]
+pub fn release_semaphore_khr(sema_object: cl_semaphore_khr) -> Result<(), cl_int> {
+    let status: cl_int = unsafe { clReleaseSemaphoreKHR(sema_object) };
+    if CL_SUCCESS != status {
+        Err(status)
+    } else {
+        Ok(())
+    }
+}
+
+#[cfg(feature = "cl_khr_semaphore")]
+pub fn retain_semaphore_khr(sema_object: cl_semaphore_khr) -> Result<(), cl_int> {
+    let status: cl_int = unsafe { clRetainSemaphoreKHR(sema_object) };
+    if CL_SUCCESS != status {
+        Err(status)
+    } else {
+        Ok(())
+    }
+}
+
 #[cfg(feature = "cl_arm_import_memory")]
 pub fn import_memory_arm(
     context: cl_context,
@@ -746,8 +1389,8 @@ pub fn shared_mem_alloc_intel(
 }
 
 #[cfg(feature = "cl_intel_unified_shared_memory")]
-pub fn mem_free_intel(context: cl_context) -> Result<(), cl_int> {
-    let status = unsafe { clMemFreeINTEL(context) };
+pub fn mem_free_intel(context: cl_context, ptr: *mut c_void) -> Result<(), cl_int> {
+    let status = unsafe { clMemFreeINTEL(context, ptr) };
     if CL_SUCCESS != status {
         Err(status)
     } else {
